@@ -16,15 +16,21 @@ to do next after a job completes
 
 from collections import defaultdict, OrderedDict
 import abc
-
 import datetime
 import os
 import pprint
-from fireworks.core.fw_config import FWConfig, NEGATIVE_FWID_CTR
+
+from monty.io import reverse_readline
+from six import add_metaclass
+
+from fireworks.fw_config import TRACKER_LINES, NEGATIVE_FWID_CTR
 from fireworks.core.fworker import FWorker
 from fireworks.utilities.dict_mods import apply_mod
-from fireworks.utilities.fw_serializers import FWSerializable, recursive_serialize, recursive_deserialize, serialize_fw
-from fireworks.utilities.fw_utilities import get_my_host, get_my_ip, NestedClassGetter, reverse_readline
+from fireworks.utilities.fw_serializers import FWSerializable, \
+    recursive_serialize, recursive_deserialize, serialize_fw
+from fireworks.utilities.fw_utilities import get_my_host, get_my_ip, \
+    NestedClassGetter
+
 
 __author__ = "Anubhav Jain"
 __credits__ = "Shyue Ping Ong"
@@ -35,10 +41,8 @@ __email__ = "ajain@lbl.gov"
 __date__ = "Feb 5, 2013"
 
 
+@add_metaclass(abc.ABCMeta)
 class FireTaskMeta(type):
-
-    __metaclass__ = abc.ABCMeta
-
     def __call__(cls, *args, **kwargs):
         o = type.__call__(cls, *args, **kwargs)
         for k in cls.required_params:
@@ -48,14 +52,14 @@ class FireTaskMeta(type):
         return o
 
 
-class FireTaskBase(dict, FWSerializable):
+@add_metaclass(FireTaskMeta)
+class FireTaskBase(defaultdict, FWSerializable):
     """
     FireTaskBase is used like an abstract class that defines a computing task
     (FireTask). All FireTasks should inherit from FireTaskBase.
 
     You can set parameters of a FireTask like you'd use a dict.
     """
-    __metaclass__ = FireTaskMeta
 
     # Specify required parameters with class variable. Consistency will be
     # checked upon init.
@@ -86,16 +90,6 @@ class FireTaskBase(dict, FWSerializable):
     def from_dict(cls, m_dict):
         return cls(m_dict)
 
-    def __getitem__(self, key):
-        """
-        Reproduce a simple defaultdict-like behavior - any unset parameters
-        return None
-        """
-        try:
-            return dict.__getitem__(self, key)
-        except KeyError:
-            return None
-
 
 class FWAction(FWSerializable):
     """
@@ -175,8 +169,9 @@ class FireWork(FWSerializable):
     STATE_RANKS = {'ARCHIVED': -1, 'DEFUSED': 0, 'WAITING': 1, 'READY': 2,
                    'RESERVED': 3, 'FIZZLED': 4, 'RUNNING': 5, 'COMPLETED': 7}
 
-    def __init__(self, tasks, spec=None, name=None, launches=None, archived_launches=None,
-                 state='WAITING', created_on=None, fw_id=None):
+    def __init__(self, tasks, spec=None, name=None, launches=None,
+                 archived_launches=None, state='WAITING', created_on=None,
+                 fw_id=None):
         """
         :param tasks: ([FireTask]) a list of FireTasks to run in sequence
         :param spec: (dict) specification of the job to run. Used by the
@@ -197,7 +192,7 @@ class FireWork(FWSerializable):
                                tasks]  # put tasks in a special location of the spec
 
         self.name = name if name else 'Unnamed FW'  # do it this way to prevent None names
-        if fw_id:
+        if fw_id is not None:
             self.fw_id = fw_id
         else:
             global NEGATIVE_FWID_CTR
@@ -280,14 +275,14 @@ class Tracker(FWSerializable, object):
 
     MAX_TRACKER_LINES = 1000
 
-    def __init__(self, filename, nlines=FWConfig().TRACKER_LINES, content=''):
+    def __init__(self, filename, nlines=TRACKER_LINES, content=''):
         if nlines > self.MAX_TRACKER_LINES:
-            raise ValueError("Tracker only supports a maximum of {} lines; you put {}.".format(self.MAX_TRACKER_LINES, nlines))
+            raise ValueError("Tracker only supports a maximum of {} lines; you put {}.".format(
+                self.MAX_TRACKER_LINES, nlines))
         self.filename = filename
         self.nlines = nlines
         self.content = content
 
-
     def track_file(self, launch_dir=None):
         """
         Reads the monitored file and returns back the last N lines
@@ -305,7 +300,7 @@ class Tracker(FWSerializable, object):
                     lines.append(l)
                     if len(lines) == self.nlines:
                         break
-            self.content='\n'.join(reversed(lines))
+            self.content = '\n'.join(reversed(lines))
 
         return self.content
 
@@ -322,6 +317,7 @@ class Tracker(FWSerializable, object):
     def __str__(self):
         return '### Filename: {}\n{}'.format(self.filename, self.content)
 
+
 class Launch(FWSerializable, object):
     """
     A Launch encapsulates data about a specific run of a FireWork on a
@@ -514,6 +510,7 @@ class Launch(FWSerializable, object):
                     return data['updated_on']
                 return data['created_on']
 
+
 class Workflow(FWSerializable):
     """
     A Workflow connects a group of FireWorks in an execution order
@@ -527,7 +524,7 @@ class Workflow(FWSerializable):
         def __init__(self, *args, **kwargs):
             super(Workflow.Links, self).__init__(*args, **kwargs)
 
-            for k, v in self.items():
+            for k, v in list(self.items()):
                 if not isinstance(v, list):
                     self[k] = [v]  # v must be list
                 if not isinstance(k, int):
@@ -539,28 +536,28 @@ class Workflow(FWSerializable):
 
         @property
         def nodes(self):
-            return self.keys()
+            return list(self.keys())
 
         @property
         def parent_links(self):
             # note: if performance of parent_links becomes an issue,
             # override delitem/setitem to update parent_links
             child_parents = defaultdict(list)
-            for (parent, children) in self.iteritems():
+            for (parent, children) in self.items():
                 for child in children:
                     child_parents[child].append(parent)
             return dict(child_parents)
 
         def to_dict(self):
             # convert to str form for Mongo, which cannot have int keys
-            return dict([(str(k), v) for (k, v) in self.iteritems()])
+            return dict([(str(k), v) for (k, v) in self.items()])
 
         def to_db_dict(self):
             # convert to str form for Mongo, which cannot have int keys
             m_dict = {
-                'links': dict([(str(k), v) for (k, v) in self.iteritems()]),
+                'links': dict([(str(k), v) for (k, v) in self.items()]),
                 'parent_links': dict(
-                    [(str(k), v) for (k, v) in self.parent_links.iteritems()]),
+                    [(str(k), v) for (k, v) in self.parent_links.items()]),
                 'nodes': self.nodes}
             return m_dict
 
@@ -576,7 +573,7 @@ class Workflow(FWSerializable):
             # to support Pickling of inner classes (for multi-job launcher's multiprocessing)
             # return a class which can return this class when called with the
             # appropriate tuple of arguments
-            state = self.items()
+            state = list(self.items())
             return (NestedClassGetter(),
                     (Workflow, self.__class__.__name__, ),
                     state)
@@ -619,7 +616,7 @@ class Workflow(FWSerializable):
 
     @property
     def fws(self):
-        return self.id_fw.values()
+        return list(self.id_fw.values())
 
     @property
     def state(self):
@@ -678,7 +675,8 @@ class Workflow(FWSerializable):
             for wf in action.detours:
                 new_updates = self._add_wf_to_fw(wf, fw_id, True)
                 if len(set(updated_ids).intersection(new_updates)) > 0:
-                    raise ValueError("Cannot use duplicated fw_ids when dynamically detouring workflows!")
+                    raise ValueError(
+                        "Cannot use duplicated fw_ids when dynamically detouring workflows!")
                 updated_ids.extend(new_updates)
 
         # add additional FireWorks
@@ -686,7 +684,8 @@ class Workflow(FWSerializable):
             for wf in action.additions:
                 new_updates = self._add_wf_to_fw(wf, fw_id, False)
                 if len(set(updated_ids).intersection(new_updates)) > 0:
-                    raise ValueError("Cannot use duplicated fw_ids when dynamically adding workflows!")
+                    raise ValueError(
+                        "Cannot use duplicated fw_ids when dynamically adding workflows!")
                 updated_ids.extend(new_updates)
 
         return list(set(updated_ids))
@@ -800,9 +799,7 @@ class Workflow(FWSerializable):
             # This part is confusing and rare - report any FIZZLED parents if allow_fizzed
             # allows us to handle FIZZLED jobs
             if fw.spec.get('_allow_fizzled_parents'):
-                parent_fws = [self.id_fw[p].to_dict() for p in
-                                   self.links.parent_links.get(fw_id, []) if
-                                   self.id_fw[p].state == 'FIZZLED']
+                parent_fws = [self.id_fw[p].to_dict() for p in self.links.parent_links.get(fw_id, []) if self.id_fw[p].state == 'FIZZLED']
                 if len(parent_fws) > 0:
                     fw.spec['_fizzled_parents'] = parent_fws
                     updated_ids.add(fw_id)
@@ -847,7 +844,7 @@ class Workflow(FWSerializable):
         """
 
         leaf_ids = []
-        for id, children in self.links.iteritems():
+        for id, children in self.links.items():
             if len(children) == 0:
                 leaf_ids.append(id)
         return leaf_ids
@@ -861,19 +858,19 @@ class Workflow(FWSerializable):
 
         # update id_fw
         new_id_fw = {}
-        for (fwid, fws) in self.id_fw.iteritems():
+        for (fwid, fws) in self.id_fw.items():
             new_id_fw[old_new.get(fwid, fwid)] = fws
         self.id_fw = new_id_fw
 
         # update the Links
         new_l = {}
-        for (parent, children) in self.links.iteritems():
+        for (parent, children) in self.links.items():
             new_l[old_new.get(parent, parent)] = [old_new.get(child, child) for
                                                   child in children]
         self.links = Workflow.Links(new_l)
 
     def to_dict(self):
-        return {'fws': [f.to_dict() for f in self.id_fw.itervalues()],
+        return {'fws': [f.to_dict() for f in self.id_fw.values()],
                 'links': self.links.to_dict(),
                 'name': self.name,
                 'metadata': self.metadata, 'updated_on': self.updated_on}
@@ -899,10 +896,10 @@ class Workflow(FWSerializable):
         m_dict['nodes'] = [self._str_fw(x) for x in nodes]
         m_dict['links'] = OrderedDict(
             [(self._str_fw(k), [self._str_fw(v) for v in a]) for k, a in
-             m_dict['links'].iteritems()])
+             m_dict['links'].items()])
         m_dict['parent_links'] = OrderedDict(
             [(self._str_fw(k), [self._str_fw(v) for v in a]) for k, a in
-             m_dict['parent_links'].iteritems()])
+             m_dict['parent_links'].items()])
         m_dict['states_list'] = '-'.join([a[0:4] for a in m_dict['states'].values()])
         return m_dict
 
