@@ -8,11 +8,13 @@ import json
 import os
 import time
 import traceback
+from collections import OrderedDict
 
 from pymongo.mongo_client import MongoClient
 from pymongo import DESCENDING, ASCENDING
 
-from fireworks.core.fw_config import FWConfig
+from fireworks.fw_config import LAUNCHPAD_LOC, CONFIG_FILE_DIR, SORT_FWS, \
+    RESERVATION_EXPIRATION_SECS, RUN_EXPIRATION_SECS, MAINTAIN_INTERVAL
 from fireworks.utilities.fw_serializers import FWSerializable
 from fireworks.core.firework import FireWork, Launch, Workflow, FWAction, \
     Tracker
@@ -32,18 +34,15 @@ __date__ = 'Jan 30, 2013'
 # TODO: can actions like complete_launch() be done as a transaction? e.g. refresh_wf() might have error...I guess at
 # least set the state to FIZZLED or ERROR and add traceback...
 
-# Note: Always use find_and_modify() for *all* database updates. Otherwise you will run into synchronization /
-# delayed write errors from Mongo. Even find_and_modify() does not guarantee zero errors, but it reduces incidents
-# considerably.
-
 
 class LaunchPad(FWSerializable):
     """
     The LaunchPad manages the FireWorks database.
     """
 
-    def __init__(self, host='localhost', port=27017, name='fireworks', username=None, password=None,
-                 logdir=None, strm_lvl=None, user_indices=None, wf_user_indices=None):
+    def __init__(self, host='localhost', port=27017, name='fireworks',
+                 username=None, password=None, logdir=None, strm_lvl=None,
+                 user_indices=None, wf_user_indices=None):
         """
 
         :param host:
@@ -65,7 +64,8 @@ class LaunchPad(FWSerializable):
         # set up logger
         self.logdir = logdir
         self.strm_lvl = strm_lvl if strm_lvl else 'INFO'
-        self.m_logger = get_fw_logger('launchpad', l_dir=self.logdir, stream_level=self.strm_lvl)
+        self.m_logger = get_fw_logger('launchpad', l_dir=self.logdir,
+                                      stream_level=self.strm_lvl)
 
         self.user_indices = user_indices if user_indices else []
         self.wf_user_indices = wf_user_indices if wf_user_indices else []
@@ -86,10 +86,12 @@ class LaunchPad(FWSerializable):
         """
         Note: usernames/passwords are exported as unencrypted Strings!
         """
-        d = {'host': self.host, 'port': self.port, 'name': self.name, 'username': self.username,
-             'password': self.password, 'logdir': self.logdir, 'strm_lvl': self.strm_lvl,
-             'user_indices': self.user_indices, 'wf_user_indices': self.wf_user_indices}
-        return d
+        return {
+            'host': self.host, 'port': self.port, 'name': self.name,
+            'username': self.username, 'password': self.password,
+            'logdir': self.logdir, 'strm_lvl': self.strm_lvl,
+            'user_indices': self.user_indices,
+            'wf_user_indices': self.wf_user_indices}
 
     @classmethod
     def from_dict(cls, d):
@@ -97,15 +99,17 @@ class LaunchPad(FWSerializable):
         strm_lvl = d.get('strm_lvl', None)
         user_indices = d.get('user_indices', [])
         wf_user_indices = d.get('wf_user_indices', [])
-        return LaunchPad(d['host'], d['port'], d['name'], d['username'], d['password'], logdir,
-                         strm_lvl, user_indices, wf_user_indices)
+        return LaunchPad(d['host'], d['port'], d['name'], d['username'],
+                         d['password'], logdir, strm_lvl, user_indices,
+                         wf_user_indices)
 
     @classmethod
     def auto_load(cls):
-        if FWConfig().LAUNCHPAD_LOC:
-            return LaunchPad.from_file(FWConfig().LAUNCHPAD_LOC)
-        elif FWConfig().CONFIG_FILE_DIR:
-            return LaunchPad.from_file(os.path.join(FWConfig().CONFIG_FILE_DIR, 'my_launchpad.yaml'))
+        if LAUNCHPAD_LOC:
+            return LaunchPad.from_file(LAUNCHPAD_LOC)
+        elif CONFIG_FILE_DIR:
+            return LaunchPad.from_file(os.path.join(CONFIG_FILE_DIR,
+                                                    'my_launchpad.yaml'))
         return LaunchPad()
 
     def reset(self, password, require_password=True):
@@ -132,7 +136,7 @@ class LaunchPad(FWSerializable):
             raise ValueError("Invalid password! Password is today's date: {}".format(m_password))
 
     def maintain(self, infinite=True, maintain_interval=None):
-        maintain_interval = maintain_interval if maintain_interval else FWConfig().MAINTAIN_INTERVAL
+        maintain_interval = maintain_interval if maintain_interval else MAINTAIN_INTERVAL
 
         while True:
             self.m_logger.info('Performing maintenance on Launchpad...')
@@ -166,7 +170,7 @@ class LaunchPad(FWSerializable):
             wf = Workflow.from_FireWork(wf)
 
         # insert the FireWorks and get back mapping of old to new ids
-        old_new = self._upsert_fws(wf.id_fw.values(), reassign_all=reassign_all)
+        old_new = self._upsert_fws(list(wf.id_fw.values()), reassign_all=reassign_all)
 
         # update the Workflow with the new ids
         wf._reassign_ids(old_new)
@@ -206,22 +210,16 @@ class LaunchPad(FWSerializable):
         if not fw_dict:
             raise ValueError('No FireWork exists with id: {}'.format(fw_id))
             # recreate launches from the launch collection
-        launches = []
-        for launch_id in fw_dict['launches']:
-            launches.append(self.get_launch_by_id(launch_id).to_dict())
 
-        fw_dict['launches'] = launches
+        fw_dict['launches'] = list(self.launches.find(
+                    {'launch_id': {"$in": fw_dict['launches']}}))
 
-        archived_launches = []
-        for launch_id in fw_dict['archived_launches']:
-            archived_launches.append(self.get_launch_by_id(launch_id).to_dict())
-
-        fw_dict['archived_launches'] = archived_launches
+        fw_dict['archived_launches'] = list(self.launches.find(
+                    {'launch_id': {"$in": fw_dict['archived_launches']}}))
 
         return FireWork.from_dict(fw_dict)
 
     def get_wf_by_fw_id(self, fw_id):
-
         """
         Given a FireWork id, give back the Workflow containing that FireWork
         :param fw_id:
@@ -229,11 +227,76 @@ class LaunchPad(FWSerializable):
         """
 
         links_dict = self.workflows.find_one({'nodes': fw_id})
-        fws = []
-        for fw_id in links_dict['nodes']:
-            fws.append(self.get_fw_by_id(fw_id))
+        fws = map(self.get_fw_by_id, links_dict["nodes"])
+        return Workflow(fws, links_dict['links'], links_dict['name'],
+                        links_dict['metadata'])
 
-        return Workflow(fws, links_dict['links'], links_dict['name'], links_dict['metadata'])
+    def get_wf_summary_dict(self, fw_id, mode="more"):
+        """
+        A much faster way to get summary information about a Workflow by
+        querying only for needed information.
+
+        Args:
+            fw_id (int): A Firework id.
+            mode (str): Choose between "more", "less" and "all" in terms of
+                quantity of information.
+
+        Returns:
+            (dict) of information about Workflow.
+        """
+        wf_fields = ["state", "created_on", "name", "nodes"]
+        fw_fields = ["state", "fw_id"]
+        launch_fields = []
+
+        if mode != "less":
+            wf_fields.append("updated_on")
+            fw_fields.extend(["name", "launches"])
+            launch_fields.append("launch_dir")
+
+        if mode == "all":
+            wf_fields = None
+
+        wf = self.workflows.find_one({"nodes": fw_id}, fields=wf_fields)
+        fw_data = []
+        id_name_map = {}
+        for fw in self.fireworks.find({"fw_id": {"$in": wf["nodes"]}},
+                                      fields=fw_fields):
+            if launch_fields:
+                fw["launches"] = list(
+                    self.launches.find({'launch_id': {"$in": fw['launches']}},
+                                       fields=launch_fields))
+            fw_data.append(fw)
+            if mode != "less":
+                id_name_map[fw["fw_id"]] = "%s--%d" % (fw["name"], fw["fw_id"])
+        wf["fw"] = fw_data
+
+        # Post process the summary dict so that it "looks" better.
+        if mode == "less":
+            wf["states_list"] = "-".join(
+                [fw["state"][:3] if fw["state"].startswith("R")
+                 else fw["state"][0] for fw in wf["fw"]])
+            del wf["nodes"]
+        elif mode == "more":
+            wf["states"] = OrderedDict()
+            wf["launch_dirs"] = OrderedDict()
+            for fw in wf["fw"]:
+                k = "%s--%d" % (fw["name"], fw["fw_id"])
+                wf["states"][k] = fw["state"]
+                wf["launch_dirs"][k] = [l["launch_dir"] for l in fw[
+                    "launches"]]
+            del wf["nodes"]
+        elif mode == "all":
+            wf["links"] = {id_name_map[int(k)]: [id_name_map[i] for i in v]
+                           for k, v in wf["links"].items()}
+            wf["nodes"] = map(id_name_map.get, wf["nodes"])
+            wf["parent_links"] = {
+                id_name_map[int(k)]: [id_name_map[i] for i in v]
+                for k, v in wf["parent_links"].items()}
+
+        del wf["_id"]
+        del wf["fw"]
+
+        return wf
 
     def get_fw_ids(self, query=None, sort=None, limit=0, count_only=False):
         """
@@ -284,23 +347,16 @@ class LaunchPad(FWSerializable):
 
         self.m_logger.debug('Updating indices...')
         self.fireworks.ensure_index('fw_id', unique=True)
-        self.fireworks.ensure_index('state')
-        self.fireworks.ensure_index('spec._category')
-        self.fireworks.ensure_index('created_on')
-        self.fireworks.ensure_index('name')
+        for f in ("state", 'spec._category', 'created_on', "name"):
+            self.fireworks.ensure_index(f)
 
         self.launches.ensure_index('launch_id', unique=True)
-        self.launches.ensure_index('state')
-        self.launches.ensure_index('time_start')
-        self.launches.ensure_index('time_end')
-        self.launches.ensure_index('host')
-        self.launches.ensure_index('ip')
-        self.launches.ensure_index('fworker.name')
+        for f in ('state', 'time_start', 'time_end', 'host', 'ip',
+                  'fworker.name'):
+            self.launches.ensure_index(f)
 
-        self.workflows.ensure_index('name')
-        self.workflows.ensure_index('created_on')
-        self.workflows.ensure_index('updated_on')
-        self.workflows.ensure_index('nodes')
+        for f in ('name', 'created_on', 'updated_on', 'nodes'):
+            self.workflows.ensure_index(f)
 
         for idx in self.user_indices:
             self.fireworks.ensure_index(idx)
@@ -315,11 +371,11 @@ class LaunchPad(FWSerializable):
         except:
             self.m_logger.debug('Database compaction failed (not critical)')
 
-
     def defuse_fw(self, fw_id):
         allowed_states = ['DEFUSED', 'WAITING', 'READY', 'FIZZLED']
-        f = self.fireworks.find_and_modify({'fw_id': fw_id, 'state': {'$in': allowed_states}},
-                                              {'$set': {'state': 'DEFUSED'}})
+        f = self.fireworks.find_and_modify(
+            {'fw_id': fw_id, 'state': {'$in': allowed_states}},
+            {'$set': {'state': 'DEFUSED'}})
 
         self._refresh_wf(self.get_wf_by_fw_id(fw_id), fw_id)
         return f
@@ -338,7 +394,6 @@ class LaunchPad(FWSerializable):
 
         self._refresh_wf(self.get_wf_by_fw_id(fw_id), fw_id)
 
-
     def reignite_wf(self, fw_id):
         wf = self.get_wf_by_fw_id(fw_id)
         for fw in wf.fws:
@@ -354,7 +409,8 @@ class LaunchPad(FWSerializable):
         # second set the state of all FWs to ARCHIVED
         wf = self.get_wf_by_fw_id(fw_id)
         for fw in wf.fws:
-            self.fireworks.find_and_modify({'fw_id': fw.fw_id}, {'$set': {'state': 'ARCHIVED'}})
+            self.fireworks.find_and_modify({'fw_id': fw.fw_id},
+                                           {'$set': {'state': 'ARCHIVED'}})
 
         self._refresh_wf(self.get_wf_by_fw_id(fw_id), fw_id)
 
@@ -390,10 +446,10 @@ class LaunchPad(FWSerializable):
 
         sortby = [("spec._priority", DESCENDING)]
 
-        if FWConfig().SORT_FWS.upper() == "FIFO":
+        if SORT_FWS.upper() == "FIFO":
             sortby.append(("created_on", ASCENDING))
 
-        elif FWConfig().SORT_FWS.upper() == "FILO":
+        elif SORT_FWS.upper() == "FILO":
             sortby.append(("created_on", DESCENDING))
 
         # Override query if fw_id defined
@@ -447,7 +503,7 @@ class LaunchPad(FWSerializable):
         for fw in self.fireworks.find({'launches': launch_id, 'state': 'RESERVED'}, {'fw_id': 1}):
             self.fireworks.find_and_modify({'fw_id': fw['fw_id']}, {'$set': {'state': 'READY'}})
 
-    def detect_unreserved(self, expiration_secs=FWConfig().RESERVATION_EXPIRATION_SECS, rerun=False):
+    def detect_unreserved(self, expiration_secs=RESERVATION_EXPIRATION_SECS, rerun=False):
         bad_launch_ids = []
         now_time = datetime.datetime.utcnow()
         cutoff_timestr = (now_time - datetime.timedelta(seconds=expiration_secs)).isoformat()
@@ -474,7 +530,7 @@ class LaunchPad(FWSerializable):
             wf = self.get_wf_by_fw_id(fw_id)
             self._refresh_wf(wf, fw_id)
 
-    def detect_lostruns(self, expiration_secs=FWConfig().RUN_EXPIRATION_SECS, fizzle=False, rerun=False, max_runtime=None):
+    def detect_lostruns(self, expiration_secs=RUN_EXPIRATION_SECS, fizzle=False, rerun=False, max_runtime=None):
         bad_launch_ids = []
         bad_fw_ids = []
         now_time = datetime.datetime.utcnow()
@@ -506,7 +562,7 @@ class LaunchPad(FWSerializable):
 
         return bad_launch_ids, bad_fw_ids
 
-    def _set_reservation_id(self, launch_id, reservation_id):
+    def set_reservation_id(self, launch_id, reservation_id):
         m_launch = self.get_launch_by_id(launch_id)
         m_launch.set_reservation_id(reservation_id)
         self.launches.find_and_modify({'launch_id': launch_id}, m_launch.to_db_dict())
@@ -607,14 +663,19 @@ class LaunchPad(FWSerializable):
         """
         Checkout the next FireWork id
         """
-        return self.fw_id_assigner.find_and_modify({}, {'$inc': {'next_fw_id': 1}})['next_fw_id']
+        try:
+            return self.fw_id_assigner.find_and_modify({}, {'$inc': {'next_fw_id': 1}})['next_fw_id']
+        except:
+            raise ValueError("Could not get next FW id! If you have not yet initialized the database, please do so by performing a database reset (e.g., lpad reset)")
 
     def get_new_launch_id(self):
         """
         Checkout the next Launch id
         """
-        return self.fw_id_assigner.find_and_modify({}, {'$inc': {'next_launch_id': 1}})[
-            'next_launch_id']
+        try:
+            return self.fw_id_assigner.find_and_modify({}, {'$inc': {'next_launch_id': 1}})['next_launch_id']
+        except:
+            raise ValueError("Could not get next launch id! If you have not yet initialized the database, please do so by performing a database reset (e.g., lpad reset)")
 
     def _upsert_fws(self, fws, reassign_all=False):
         old_new = {} # mapping between old and new FireWork ids
@@ -655,7 +716,7 @@ class LaunchPad(FWSerializable):
         old_new = self._upsert_fws(updated_fws)
         wf._reassign_ids(old_new)
         # redo the links - note that if you don't search across all keys, you can get errors
-        self.workflows.find_and_modify({'nodes': {'$in': wf.id_fw.keys()}}, wf.to_db_dict())
+        self.workflows.find_and_modify({'nodes': {'$in': list(wf.id_fw.keys())}}, wf.to_db_dict())
 
 
     def _steal_launches(self, thief_fw):
