@@ -7,19 +7,26 @@ A runnable script for managing a FireWorks database (a command-line interface to
 from argparse import ArgumentParser
 import os
 import webbrowser
-from pymongo import DESCENDING, ASCENDING
 import time
-from fireworks.core.fw_config import FWConfig
-from fireworks.core.launchpad import LaunchPad
-from fireworks.core.firework import Workflow, FireWork
 import ast
 import json
 import datetime
 import traceback
+
+from pymongo import DESCENDING, ASCENDING
+import yaml
+
+from fireworks.fw_config import RESERVATION_EXPIRATION_SECS, \
+    RUN_EXPIRATION_SECS, PW_CHECK_NUM, MAINTAIN_INTERVAL, CONFIG_FILE_DIR, \
+    LAUNCHPAD_LOC
+from fireworks.core.launchpad import LaunchPad
+from fireworks.core.firework import Workflow, FireWork
 from fireworks import __version__ as FW_VERSION
 from fireworks import FW_INSTALL_DIR
 from fireworks.user_objects.firetasks.script_task import ScriptTask
-from fireworks.utilities.fw_serializers import DATETIME_HANDLER
+from fireworks.utilities.fw_serializers import DATETIME_HANDLER, recursive_dict
+from six.moves import input
+
 
 __author__ = 'Anubhav Jain'
 __credits__ = 'Shyue Ping Ong'
@@ -34,15 +41,15 @@ DEFAULT_LPAD_YAML = "my_launchpad.yaml"
 
 
 def pw_check(ids, args, skip_pw=False):
-    if len(ids) > FWConfig().PW_CHECK_NUM and not skip_pw:
+    if len(ids) > PW_CHECK_NUM and not skip_pw:
         m_password = datetime.datetime.now().strftime('%Y-%m-%d')
         if not args.password:
-            if raw_input('Are you sure? This will modify {} entries. (Y/N)'.format(len(ids)))[0].upper() == 'Y':
+            if input('Are you sure? This will modify {} entries. (Y/N)'.format(len(ids)))[0].upper() == 'Y':
                 args.password=datetime.datetime.now().strftime('%Y-%m-%d')
             else:
                 raise ValueError('Operation aborted by user.')
         if args.password != m_password:
-            raise ValueError("Modifying more than {} entries requires setting the --password parameter! (Today's date, e.g. 2012-02-25)".format(FWConfig().PW_CHECK_NUM))
+            raise ValueError("Modifying more than {} entries requires setting the --password parameter! (Today's date, e.g. 2012-02-25)".format(PW_CHECK_NUM))
 
     return ids
 
@@ -85,7 +92,7 @@ def get_lp(args):
             return LaunchPad(logdir=args.logdir, strm_lvl=args.loglvl)
     except:
         traceback.print_exc()
-        err_message = 'FireWorks was not able to connect to MongoDB. The database file specified was {}.'.format(args.launchpad_file)
+        err_message = 'FireWorks was not able to connect to MongoDB. Is the server running? The database file specified was {}.'.format(args.launchpad_file)
         if not args.launchpad_file:
             err_message += ' Type "lpad init" if you would like to set up a file that specifies location and credentials of your Mongo database (otherwise use default localhost configuration).'
         raise ValueError(err_message)
@@ -102,7 +109,7 @@ def init_yaml(args):
     print("Please supply the following configuration values")
     print("(press Enter if you want to accept the defaults)\n")
     for k, v in fields:
-        val = raw_input("Enter {} (default: {}) : ".format(k, v))
+        val = input("Enter {} (default: {}) : ".format(k, v))
         doc[k] = val if val else v
     doc["port"] = int(doc["port"])  # enforce the port as an int
     with open(args.config_file, "w") as f:
@@ -114,7 +121,7 @@ def init_yaml(args):
 def reset(args):
     lp = get_lp(args)
     if not args.password:
-        if raw_input('Are you sure? This will RESET {} workflows and all data. (Y/N)'.format(lp.workflows.count()))[0].upper() == 'Y': args.password=datetime.datetime.now().strftime('%Y-%m-%d')
+        if input('Are you sure? This will RESET {} workflows and all data. (Y/N)'.format(lp.workflows.count()))[0].upper() == 'Y': args.password=datetime.datetime.now().strftime('%Y-%m-%d')
         else:
             raise ValueError('Operation aborted by user.')
     lp.reset(args.password)
@@ -184,7 +191,7 @@ def get_fws(args):
     if len(fws) == 1:
         fws = fws[0]
 
-    print json.dumps(fws, default=DATETIME_HANDLER, indent=4)
+    print(args.output(fws))
 
 
 def get_wfs(args):
@@ -214,28 +221,16 @@ def get_wfs(args):
         sort = None
 
     ids = lp.get_wf_ids(query, sort, args.max, count_only=args.display_format == 'count')
-    wfs = []
     if args.display_format == 'ids':
         wfs = ids
     elif args.display_format == 'count':
         wfs = [ids]
     else:
-        for id in ids:
-            wf = lp.get_wf_by_fw_id(id)
-            d = wf.to_display_dict()
-            if args.display_format == 'more' or args.display_format == 'less':
-                del d['name']
-                del d['parent_links']
-                del d['nodes']
-                del d['links']
-                del d['metadata']
-            if args.display_format == 'less':
-                del d['states']
-                del d['launch_dirs']
-                del d['updated_on']
-            if args.display_format == 'more' or args.display_format == 'all':
-                del d['states_list']
+        wfs = []
+        for i in ids:
+            d = lp.get_wf_summary_dict(i, args.display_format)
             wfs.append(d)
+
     if len(wfs) == 1:
         wfs = wfs[0]
 
@@ -245,9 +240,9 @@ def get_wfs(args):
         t = PrettyTable(headers)
         for d in wfs:
             t.add_row([d.get(k) for k in headers])
-        print t
+        print(t)
     else:
-        print json.dumps(wfs, default=DATETIME_HANDLER, indent=4)
+        print(args.output(wfs))
 
 
 def detect_lostruns(args):
@@ -259,7 +254,7 @@ def detect_lostruns(args):
 
 def detect_unreserved(args):
     lp = get_lp(args)
-    print lp.detect_unreserved(expiration_secs=args.time, rerun=args.rerun)
+    print(lp.detect_unreserved(expiration_secs=args.time, rerun=args.rerun))
 
 
 def tuneup(args):
@@ -275,6 +270,7 @@ def defuse(args):
         lp.m_logger.debug('Processed fw_id: {}'.format(f))
     lp.m_logger.info('Finished defusing {} FWs'.format(len(fw_ids)))
 
+
 def archive(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args, wf_mode=True)
@@ -283,6 +279,7 @@ def archive(args):
         lp.m_logger.debug('Processed fw_id: {}'.format(f))
     lp.m_logger.info('Finished archiving {} FWs'.format(len(fw_ids)))
 
+
 def reignite(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args, wf_mode=True)
@@ -291,6 +288,7 @@ def reignite(args):
         lp.m_logger.debug('Processed Workflow with fw_id: {}'.format(f))
     lp.m_logger.info('Finished reigniting {} Workflows'.format(len(fw_ids)))
 
+
 def defuse_fws(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args)
@@ -299,6 +297,7 @@ def defuse_fws(args):
         lp.m_logger.debug('Processed fw_id: {}'.format(f))
     lp.m_logger.info('Finished defusing {} FWs'.format(len(fw_ids)))
 
+
 def reignite_fws(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args)
@@ -307,6 +306,7 @@ def reignite_fws(args):
         lp.m_logger.debug('Processed fw_id: {}'.format(f))
     lp.m_logger.info('Finished reigniting {} FWs'.format(len(fw_ids)))
 
+
 def rerun_fws(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args)
@@ -315,6 +315,7 @@ def rerun_fws(args):
         lp.m_logger.debug('Processed fw_id: {}'.format(f))
     lp.m_logger.info('Finished setting {} FWs to rerun'.format(len(fw_ids)))
 
+
 def refresh(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args, wf_mode=True)
@@ -324,6 +325,7 @@ def refresh(args):
         lp.m_logger.debug('Processed Workflow with fw_id: {}'.format(f))
     lp.m_logger.info('Finished refreshing {} Workflows'.format(len(fw_ids)))
 
+
 def set_priority(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args)
@@ -332,6 +334,7 @@ def set_priority(args):
         lp.m_logger.debug("Processed fw_id {}".format(f))
     lp.m_logger.info("Finished setting priorities of {} FWs".format(len(fw_ids)))
 
+
 def webgui(args):
     lp = get_lp(args)
     os.environ.setdefault("DJANGO_SETTINGS_MODULE", "fireworks.base_site.settings")
@@ -346,6 +349,7 @@ def webgui(args):
         webbrowser.open("http://{}:{}".format(args.host, args.port))
     p1.join()
 
+
 def add_scripts(args):
     lp = get_lp(args)
     args.names = args.names if args.names else [None] * len(args.scripts)
@@ -359,6 +363,7 @@ def add_scripts(args):
 
     lp.add_wf(Workflow(fws, links, args.wf_name))
 
+
 def recover_offline(args):
     lp = get_lp(args)
     failed_fws = []
@@ -371,6 +376,7 @@ def recover_offline(args):
     if failed_fws:
         lp.m_logger.info("FAILED to recover offline fw_ids: {}".format(failed_fws))
 
+
 def forget_offline(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args)
@@ -380,6 +386,7 @@ def forget_offline(args):
 
     lp.m_logger.info('Finished forget_offine, processed {} FWs'.format(len(fw_ids)))
 
+
 def track_fws(args):
     lp = get_lp(args)
     fw_ids = parse_helper(lp, args, skip_pw=True)
@@ -402,22 +409,40 @@ def track_fws(args):
                 first_print = False
             else:
                 output.insert(0, '>------<')
-            print '\n'.join(output)
+            print('\n'.join(output))
+
 
 def version(args):
-    print 'FireWorks version:', FW_VERSION
-    print 'located in:', FW_INSTALL_DIR
+    print('FireWorks version:', FW_VERSION)
+    print('located in:', FW_INSTALL_DIR)
+
 
 def maintain(args):
     lp = get_lp(args)
     lp.maintain(args.infinite, args.maintain_interval)
 
+
+def get_output_func(format):
+    if format == "json":
+        return lambda x: json.dumps(x, default=DATETIME_HANDLER,
+                                    indent=4)
+    else:
+        return lambda x: yaml.dump(recursive_dict(x), default_flow_style=False)
+
+
 def lpad():
     m_description = 'This script is used for creating and managing a FireWorks database (LaunchPad). For a list of ' \
                     'available commands, type "lpad -h". For more help on a specific command, ' \
                     'type "lpad <command> -h".'
 
     parser = ArgumentParser(description=m_description)
+
+    parser.add_argument("-o", "--output", choices=["json", "yaml"],
+                        default="json", type=str.lower,
+                        help="Set output dispaly format to either json or "
+                             "YAML. YAML is easier to read for long "
+                             "documents. JSON is the default.")
+
     subparsers = parser.add_subparsers(help='command', dest='command')
 
     init_parser = subparsers.add_parser(
@@ -447,30 +472,32 @@ def lpad():
 
     state_args = ['-s', '--state']
     state_kwargs = {"type": str.upper, "help": "Select by state.",
-                    "choices": ["ARCHIVED", "DEFUSED", "WAITING", "READY",
-                                "RESERVED", "FIZZLED", "RUNNING",
-                                "COMPLETED"]}
+                    "choices": FireWork.STATE_RANKS.keys()}
     disp_args = ['-d', '--display_format']
     disp_kwargs = {"type": str, "help": "Display format.",
                    "choices": ["all", "more", "less", "ids", "count"]}
 
     query_args = ["-q", "--query"]
-    query_kwargs = {"help": 'Query (enclose pymongo-style dict in single-quotes, e.g. \'{"state":"COMPLETED"}\')'}
+    query_kwargs = {"help": 'Query (enclose pymongo-style dict in '
+                            'single-quotes, e.g. \'{"state":"COMPLETED"}\')'}
 
-    get_fw_parser = subparsers.add_parser('get_fws', help='get information about FireWorks')
+    get_fw_parser = subparsers.add_parser(
+        'get_fws', help='get information about FireWorks')
     get_fw_parser.add_argument(*fw_id_args, **fw_id_kwargs)
     get_fw_parser.add_argument('-n', '--name', help='get FWs with this name')
     get_fw_parser.add_argument(*state_args, **state_kwargs)
     get_fw_parser.add_argument(*query_args, **query_kwargs)
     get_fw_parser.add_argument(*disp_args, **disp_kwargs)
-    get_fw_parser.add_argument('-m', '--max', help='limit results', default=0, type=int)
+    get_fw_parser.add_argument('-m', '--max', help='limit results', default=0,
+                               type=int)
     get_fw_parser.add_argument('--sort', help='Sort results',
                                choices=["created_on", "updated_on"])
     get_fw_parser.add_argument('--rsort', help='Reverse sort results',
                                choices=["created_on", "updated_on"])
     get_fw_parser.set_defaults(func=get_fws)
 
-    get_wf_parser = subparsers.add_parser('get_wfs', help='get information about Workflows')
+    get_wf_parser = subparsers.add_parser(
+        'get_wfs', help='get information about Workflows')
     get_wf_parser.add_argument(*fw_id_args, **fw_id_kwargs)
     get_wf_parser.add_argument('-n', '--name', help='get WFs with this name')
     get_wf_parser.add_argument(*state_args, **state_kwargs)
@@ -493,19 +520,19 @@ def lpad():
     rerun_fws_parser.add_argument('-n', '--name', help='name')
     rerun_fws_parser.add_argument(*state_args, **state_kwargs)
     rerun_fws_parser.add_argument(*query_args, **query_kwargs)
-    rerun_fws_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    rerun_fws_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     rerun_fws_parser.set_defaults(func=rerun_fws)
 
     reservation_parser = subparsers.add_parser('detect_unreserved', help='Find launches with stale reservations')
     reservation_parser.add_argument('--time', help='expiration time (seconds)',
-                                    default=FWConfig().RESERVATION_EXPIRATION_SECS, type=int)
+                                    default=RESERVATION_EXPIRATION_SECS, type=int)
     reservation_parser.add_argument('--rerun', help='cancel and rerun expired reservations', action='store_true')
     reservation_parser.set_defaults(func=detect_unreserved)
 
     fizzled_parser = subparsers.add_parser('detect_lostruns',
                                            help='Find launches that have FIZZLED')
     fizzled_parser.add_argument('--time', help='expiration time (seconds)',
-                                default=FWConfig().RUN_EXPIRATION_SECS,
+                                default=RUN_EXPIRATION_SECS,
                                 type=int)
     fizzled_parser.add_argument('--fizzle', help='mark lost runs as fizzled', action='store_true')
     fizzled_parser.add_argument('--rerun', help='rerun lost runs', action='store_true')
@@ -518,7 +545,7 @@ def lpad():
     defuse_parser.add_argument('-n', '--name', help='name')
     defuse_parser.add_argument(*state_args, **state_kwargs)
     defuse_parser.add_argument(*query_args, **query_kwargs)
-    defuse_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    defuse_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     defuse_parser.set_defaults(func=defuse)
 
     archive_parser = subparsers.add_parser('archive', help='archive an entire Workflow (irreversible)')
@@ -526,7 +553,7 @@ def lpad():
     archive_parser.add_argument('-n', '--name', help='name')
     archive_parser.add_argument(*state_args, **state_kwargs)
     archive_parser.add_argument(*query_args, **query_kwargs)
-    archive_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    archive_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     archive_parser.set_defaults(func=archive)
 
     reignite_parser = subparsers.add_parser('reignite', help='reignite (un-cancel) an entire Workflow')
@@ -534,7 +561,7 @@ def lpad():
     reignite_parser.add_argument('-n', '--name', help='name')
     reignite_parser.add_argument(*state_args, **state_kwargs)
     reignite_parser.add_argument(*query_args, **query_kwargs)
-    reignite_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    reignite_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     reignite_parser.set_defaults(func=reignite)
 
     defuse_fw_parser = subparsers.add_parser('defuse_fws', help='cancel (de-fuse) a single FireWork')
@@ -542,7 +569,7 @@ def lpad():
     defuse_fw_parser.add_argument('-n', '--name', help='name')
     defuse_fw_parser.add_argument(*state_args, **state_kwargs)
     defuse_fw_parser.add_argument(*query_args, **query_kwargs)
-    defuse_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    defuse_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     defuse_fw_parser.set_defaults(func=defuse_fws)
 
     reignite_fw_parser = subparsers.add_parser('reignite_fws', help='reignite (un-cancel) a single FireWork')
@@ -550,12 +577,12 @@ def lpad():
     reignite_fw_parser.add_argument('-n', '--name', help='name')
     reignite_fw_parser.add_argument(*state_args, **state_kwargs)
     reignite_fw_parser.add_argument(*query_args, **query_kwargs)
-    reignite_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    reignite_fw_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     reignite_fw_parser.set_defaults(func=reignite_fws)
 
     maintain_parser = subparsers.add_parser('maintain', help='Run database maintenance')
     maintain_parser.add_argument('--infinite', help='loop infinitely', action='store_true')
-    maintain_parser.add_argument('--maintain_interval', help='sleep time between maintenance loops (infinite mode)', default=FWConfig().MAINTAIN_INTERVAL, type=int)
+    maintain_parser.add_argument('--maintain_interval', help='sleep time between maintenance loops (infinite mode)', default=MAINTAIN_INTERVAL, type=int)
     maintain_parser.set_defaults(func=maintain)
 
     tuneup_parser = subparsers.add_parser('tuneup',
@@ -567,7 +594,7 @@ def lpad():
     refresh_parser.add_argument('-n', '--name', help='name')
     refresh_parser.add_argument(*state_args, **state_kwargs)
     refresh_parser.add_argument(*query_args, **query_kwargs)
-    refresh_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    refresh_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     refresh_parser.set_defaults(func=refresh)
 
     priority_parser = subparsers.add_parser('set_priority', help='modify the priority of one or more FireWorks')
@@ -576,7 +603,7 @@ def lpad():
     priority_parser.add_argument('-n', '--name', help='name')
     priority_parser.add_argument(*state_args, **state_kwargs)
     priority_parser.add_argument(*query_args, **query_kwargs)
-    priority_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(FWConfig().PW_CHECK_NUM))
+    priority_parser.add_argument('--password', help="Today's date, e.g. 2012-02-25. Password or positive response to input prompt required when modifying more than {} entries.".format(PW_CHECK_NUM))
     priority_parser.set_defaults(func=set_priority)
 
     version_parser = subparsers.add_parser(
@@ -585,10 +612,10 @@ def lpad():
     version_parser.set_defaults(func=version)
 
     parser.add_argument('-l', '--launchpad_file', help='path to LaunchPad file containing central DB connection info',
-                        default=FWConfig().LAUNCHPAD_LOC)
+                        default=LAUNCHPAD_LOC)
     parser.add_argument('-c', '--config_dir',
                         help='path to a directory containing the LaunchPad file (used if -l unspecified)',
-                        default=FWConfig().CONFIG_FILE_DIR)
+                        default=CONFIG_FILE_DIR)
     parser.add_argument('--logdir', help='path to a directory for logging')
     parser.add_argument('--loglvl', help='level to print log messages', default='INFO')
     parser.add_argument('-s', '--silencer', help='shortcut to mute log messages', action='store_true')
@@ -631,6 +658,9 @@ def lpad():
     trackfw_parser.set_defaults(func=track_fws)
 
     args = parser.parse_args()
+
+    args.output = get_output_func(args.output)
+
     args.func(args)
 
 
